import asyncdispatch, httpclient, os, osproc, strutils, times, unittest
import ./testutil
import ../../main/nim/strawbosspkg/configuration
import ../../main/nim/strawbosspkg/server
import ../../main/nim/strawbosspkg/private/util

suite "strawboss server":

  # suite setup code
  let cfgFilePath = "src/test/json/strawboss.config.json"
  let cfg = loadStrawBossConfig(cfgFilePath)

  discard startProcess("./strawboss", ".", @["serve", "-c", cfgFilePath], loadEnv(), {poUsePath})

  let http = newHttpClient()
  let apiBase = "http://localhost:8180/api"

  let testuser = UserRef( # note: needs to correspond to an actual user
    name: "bob@builder.com",
    hashedPwd: "$2a$11$lVZ9U4optQMhzPh0E9A7Yu6XndXblUF3gCa.zmEvJy4F.4C4718b.")

  # give the server time to spin up
  sleep(100)

  ## UNIT TESTS

  test "can validate hashed pwd":

    check validatePwd(testuser, "password")

  test "can detect invalid pwds":
    check(not validatePwd(testuser, "Password"))

  test "can make and extract a JWT token from a session":
    let session = newSession(testuser)
    let tok = toJWT(cfg, session)

    check:
      fromJWT(cfg, tok) == session

  test "can ping":
    let resp = http.get(apiBase & "/ping")
    check:
      resp.status.startsWith("200")
      resp.body == "\"pong\""

  test "can fail auth":
    let resp = http.get(apiBase & "/auth-token?username=bob@builder.com&password=notpassword")
    check:
      resp.status.startsWith("401")

  test "can auth":
    let resp = http.get(apiBase & "/auth-token?username=bob@builder.com&password=password")
    check:
      resp.status.startsWith("200")

  # suite tear-down
  try: discard http.post(apiBase & "/service/debug/stop")
  except: discard ""
